﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Windows;

using DevExpress.Data;
using DevExpress.Mvvm;
using DevExpress.Mvvm.UI;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 통지 서비스
        /// </summary>
        private NotificationService service;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            DataTemplate template = FindResource("CustomDataTemplateKey") as DataTemplate;

            this.service = new NotificationService();

            this.service.ApplicationId                     = "TestProject";
            this.service.ApplicationName                   = "TestProject";
            this.service.UseWin8NotificationsIfAvailable   = true;
            this.service.CreateApplicationShortcut         = true;
            this.service.CustomNotificationTemplate        = template;
            this.service.CustomNotificationDuration        = TimeSpan.FromSeconds(1);
            this.service.CustomNotificationPosition        = NotificationPosition.BottomRight;
            this.service.CustomNotificationVisibleMaxCount = 10;

            ShellHelper.TryCreateShortcut(this.service.ApplicationId, this.service.ApplicationName);

            Closing               += Window_Closing;
            this.sendButton.Click += sendButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우 닫을 경우 처리하기 - Window_Closing(sender, e)

        /// <summary>
        /// 윈도우 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            ShellHelper.TryRemoveShortcut(this.service.ApplicationName);
        }

        #endregion
        #region 토스트 메시지 보내기 버튼 클릭시 처리하기 - sendButton_Click(sender, e)

        /// <summary>
        /// 토스트 메시지 보내기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sendButton_Click(object sender, RoutedEventArgs e)
        {
            NotificationModel model = new NotificationModel();

            model.Caption = "알림";
            model.Content = "등록된 사용자가 없습니다.";
            model.Time    = $"시각 : {DateTime.Now}";

            INotification notification = this.service.CreateCustomNotification(model);

            notification.ShowAsync().ContinueWith(resultTask => OnToastMessageClicked(resultTask, null));
        }

        #endregion
        #region 토스트 메시지 클릭시 처리하기 - OnToastMessageClicked(resultTask, parameter)

        /// <summary>
        /// 토스트 메시지 클릭시 처리하기
        /// </summary>
        /// <param name="resultTask">결과 태스크</param>
        /// <param name="parameter">매개 변수</param>
        private void OnToastMessageClicked(Task<NotificationResult> resultTask, object parameter)
        {
            NotificationResult result = resultTask.Result;

            switch(result)
            {
                case NotificationResult.Activated :

                    Process.Start("https://icodebroker.tistory.com");

                    break;
            }
        }

        #endregion
    }
}